// Black Box game
// $Id: BlackBox.java 22 2006-09-05 21:58:20Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This is the top level class for the Black Box game applet/application

package com.brouhaha.blackbox;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BlackBox extends JApplet
{
    public String getAppletInfo ()
    {
    	return "BlackBox applet by Eric Smith";
    }
    
    private static boolean application = false;

    private Board board;
    private JPanel vpanel;
    private JButton guess_button;
    private JButton restart_button;
    private JButton quit_button;

    // Applet initialization.
    public void init ()
    {
	int size, atoms;

	try
	    {
		size = Integer.parseInt (getParameter ("size"));
	    }
	catch (java.lang.NumberFormatException ex)
	    {
		size = 8;
	    }

	try
	    {
		atoms = Integer.parseInt (getParameter ("atoms"));
	    }
	catch (java.lang.NumberFormatException ex)
	    {
		atoms = 4;
	    }

	System.out.printf ("size = %d, atoms = %d\n", size, atoms);
	play_game (size, atoms);
    }

    // Play a game of Black Box!
    private void play_game (int size, int atoms)
    {
    	board = new Board (size, atoms);
    	add (board, BorderLayout.CENTER);
    	
    	vpanel = new JPanel ();
    	vpanel.setLayout (new BoxLayout (vpanel, BoxLayout.Y_AXIS));
    	add (vpanel, BorderLayout.EAST);

    	vpanel.add (board.score);

    	guess_button = new JButton ("Try guesses");
    	guess_button.addActionListener (new ActionListener ()
	    {
		public void actionPerformed (ActionEvent ev)
		{
		    board.submit_guesses ();
		}
	    });
    	vpanel.add (guess_button);
	
    	restart_button = new JButton ("New game");
    	restart_button.addActionListener (new ActionListener ()
	    {
		public void actionPerformed (ActionEvent ev)
		{
		    board.reset ();
		}
		});
    	vpanel.add (restart_button);
	
    	if (application)
	    {
    		quit_button = new JButton ("Quit");
    		quit_button.addActionListener (new ActionListener ()
		    {
			public void actionPerformed (ActionEvent ev)
			{
			    destroy ();
			    System.exit (0);
			}
		    });
    		vpanel.add (quit_button);
	    }
    }

    // Other applet functions, not essential for this game.
    public void start ()   { }
    public void stop ()    { }
    public void destroy () { }

    private static void usage ()
    {
	System.err.printf ("Usage: BlackBox [size atoms]\n");
    }

    // Main program when running as application.  Just acts as a wrapper
    // for the applet.
    public static void main (String args [])
    {
	int size = 8;
	int atoms = 4;

	switch (args.length)
	    {
	    case 0:
		break;
	    case 2:
		try
		    {
			size = Integer.parseInt (args [0]);
			atoms = Integer.parseInt (args [1]);
		    }
		catch (java.lang.NumberFormatException ex)
		    {
			usage ();
			System.exit (1);
		    }
		break;
	    default:
		System.err.printf ("Invalid argument count %d\n", args.length);
		usage ();
		System.exit (1);
	    }


	application = true;
	JFrame f = new JFrame ("Black Box");
	BlackBox applet = new BlackBox ();
	f.getContentPane ().add ("Center", applet);
	applet.play_game (size, atoms);
	f.pack ();
	f.setSize (new Dimension (400, 400));
	f.setVisible (true);  // Older versions of Java used f.show (), which
	                      // is now deprecated.
    }

    // We don't need serialization, but the following avoids
    // warnings from Eclipse.
    private static final long serialVersionUID = 1L;
}
