// Black Box game
// $Id: Coordinate.java 16 2006-09-03 07:55:13Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This class represents a coordinate on the Black Box board, or on the
// edge and corner squares that are adjacent to the board.  The coordinate
// has x and y components that range from 0 (left or top edge) to size+1
// (right or bottom edge).

// The size of the board must be established by a specific constructor
// call.  Aside from that constructor, additional constructors are provided
// to create a Coordinate from x and y components or another Coordinate.
// Coordinates are considered to be immutable.

// Predicate functions are provided to determine equality of two coordinates,
// whether a single coordinate is associated with a specific edge of the
// board, any edge, or a corner square.

// A function is provided to obtain the next coordinate in a given heading.

package com.brouhaha.blackbox;

class Coordinate
{
    static int width;
    static int height;

    final int x;
    final int y;

    // Create a coordinate for a square board.  This needs to be used
    // once to initialize the class.
    Coordinate (int width, int height, int x, int y)
    {
	Coordinate.width = width;
	Coordinate.height = height;
	this.x = 0;
	this.y = 0;
    }

    // Create a Coordinate from x and y components.
    Coordinate (int x, int y)
    {
	this.x = x;
	this.y = y;
    }

    boolean equal (Coordinate coord)
    {
	return (coord.x == this.x) && (coord.y == this.y);
    }

    Coordinate next_coord (Heading heading)
    {
    	return new Coordinate (this.x + heading.x_dir,
			       this.y + heading.y_dir);
    }

    boolean top_edge ()
    {
	return (y == 0);
    }

    boolean bottom_edge ()
    {
	return (y == height + 1);
    }

    boolean left_edge ()
    {
	return (x == 0);
    }

    boolean right_edge ()
    {
	return (x == width + 1);
    }

    boolean edge ()
    {
	return (top_edge () || bottom_edge () || left_edge () || right_edge ());
    }

    boolean corner ()
    {
	return ((top_edge () || bottom_edge ()) && (left_edge () || right_edge ()));
    }
}
