// Black Box game
// $Id: Heading.java 2 2006-08-30 00:45:51Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This enumeration type represents a compass heading (north, south, east,
// or west), and provides the ability to access components representing
// the unit vector in the given direction (e.g., linear combination of
// unit vectors east and sount), and functions to return headings that
// are 90 degrees clockwise or counterclockwise of a heading.

package com.brouhaha.blackbox;

enum Heading
{
    north ( 0, -1,   0),   // Define actual enumeration values.  Constructor
    east  ( 1,  0,  90),   // arguments define unit vector components and
    south ( 0,  1, 180),   // angle in degrees.
    west  (-1,  0, 270);
	
    int x_dir;       // Instance variables store unit vector components.
    int y_dir;
    double angle;

    // Constructor for internal use only
    private Heading (int x_dir, int y_dir, double angle)
    {
	this.x_dir = x_dir;
	this.y_dir = y_dir;
	this.angle = angle;
    }

    // Return the heading that is 90 degrees clockwise from this one.
    public Heading clockwise ()
    {
	switch (this)
	    {
	    case east:   return south;
	    case south:  return west;
	    case west:   return north;
	    case north:  return east;
	    }
	throw new Error ();  // Can't get here, as the switch statement
	                     // enumerates all four possible headings.  But
	                     // if we don't do anything here, a compiler
	                     // error results, as the compiler isn't smart
                             // enough to determine that this is unreachable.
    }

    // Return the heading that is 90 degrees counterclockwise from this one.
    public Heading counterclockwise ()
    {
	switch (this)
	    {
	    case east:   return north;
	    case north:  return west;
	    case west:   return south;
	    case south:  return east;
	    }
	throw new Error ();  // Can't get here, as the switch statement
	                     // enumerates all four possible headings.  But
	                     // if we don't do anything here, a compiler
	                     // error results, as the compiler isn't smart
                             // enough to determine that this is unreachable.
    }
}
