// Black Box game
// $Id: Ray.java 2 2006-08-30 00:45:51Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This class represents a ray that is fired into the black box from an
// edge square.  The ray records its initial coordinate of entry into
// the black box (along the edge), and its final coordinate of exit from the
// black box (along an edge), or that it hit an atom.

// A predicate function is available to determine whether the result of
// firing a ray was a reflection (exit to the same edge square as its origin).

// Unique endpoint numbers are assigned to pairs of rays that exit the box
// without reflection.  (The rules are such that paths are unique and
// bidirectional.)  The endpoint numbers are assigned by the process_ray()
// method of class Square or its subclasses based on the counter
// ray_endpoint_number in the Board class.

package com.brouhaha.blackbox;

class Ray
{
    Coordinate initial_coord;
    Coordinate final_coord;  // final location (unless hit)
    boolean hit;
    int endpoint_number;

    // variables used during processing of ray through board
    Coordinate current_coord;
    Heading heading;

    Ray (Coordinate initial_coord, Heading initial_heading)
    {
	this.initial_coord = initial_coord;
	this.current_coord = initial_coord;
	this.heading = initial_heading;
	hit = false;
    }

    boolean reflected ()
    {
	return ((! hit) &&
		(final_coord != null) &&
		final_coord.equal (initial_coord));
    }
}
